<?
// ******************************************************************************
// php dynamic resizer with cache
// by Fabrizio Zellini <http://fabrizio.zellini.org>
// Distributed under the GNU General Public Licence
// ******************************************************************************

// limits for the resizer
define ("MAX_WIDTH",1280);
define ("MAX_HEIGHT",1024);

function genRandString(){
    $chset="abcdefghijklmnopqrstuvwxyz0123456789";
    $tok="";
    srand();
    for ($i=0;$i<16;$i++) {
      $tok.=$chset[rand(0,strlen($chset))];
    }
    return $tok;
}

function grab ($url, &$contentType) {
    $curl = curl_init();
    // set URL and other appropriate options
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_FAILONERROR, true);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
    
    $ua =  getParameterValue ($_SERVER,"HTTP_USER_AGENT");
    if ($ua) {
      curl_setopt($curl, CURLOPT_USERAGENT, $ua);
    }
    
    $referer = getParameterValue ($_SERVER,"HTTP_REFERER");
    if ($referer) {
      curl_setopt($curl, CURLOPT_REFERER, $referer);
    }
    
    // curl_setopt($curl, CURLOPT_HEADER, true);
    curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 15);
    curl_setopt($curl, CURLOPT_TIMEOUT, 15);
    curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
    // get response
    $response = curl_exec($curl);
    $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    $contentType = curl_getinfo($curl, CURLINFO_CONTENT_TYPE );
    curl_close ($curl);
    if ($httpCode!=200 && $httpCode!=201){
      error_log("httpCode is [$httpCode] for [$url]");
      return null;
    }
    // parse response
    return $response;
}



/**
 * Parse an expression like 200x | x200 | 300x200 | 300x200c50
 *
 * @param string $expr
 * @param number $returnWidth
 * @param number $returnHeight
 * @param number $returnCrop
 */
function parseResizeExpr($expr, &$returnWidth, &$returnHeight,&$returnCrop){
  
  // crop specified ? (at end of resizeExpr )
  $idx = strpos($expr, 'c');
  if ($idx!==false) {
    $returnCrop = substr($expr,$idx+1);
    
    if ($returnCrop=="") $returnCrop=50;
    
    $expr = substr($expr,0,$idx);
  }
  
  $idx = strpos($expr, 'x');
  // starts with x
  if ($idx === 0) {
    $returnWidth = 0;
    $returnHeight = substr($expr, 1);
  }
  // ends with x
  elseif ($idx === strlen($expr) - 1) {
    $returnWidth = substr($expr, 0, strlen($expr)-1);
    $returnHeight = 0;      
  }
  // contains x
  elseif ($idx !== false) {
    $returnWidth = substr($expr, 0, $idx);
    $returnHeight = substr($expr, $idx +1);
  } else {
    trigger_error('Invalid image dimensions', E_USER_ERROR);
  }

}

function resizeAndOuput($resizeExpr, $imageurl,$expire=CACHETIME) {

  $cleanbase = realpath(BASE);  

  //echo $fullname;die();

  $cacheKey = md5($imageurl . '|' . $resizeExpr );
  $cacheDir = realpath(dirname(__FILE__)) . '/picscache/' . $resizeExpr . '/';

  // create cache dir if no exists
  if (!file_exists($cacheDir)){
    mkdir(realpath(dirname(__FILE__)) . '/picscache', 0777);
    mkdir($cacheDir, 0777);
    chmod ($cacheDir,0777);
  }
  $cacheFile = $cacheDir . $cacheKey;

  if ( file_exists($cacheFile) && ((time()-filemtime($cacheFile))<$expire) ) {
    // Output the cached one
    $expire = $expire - (time()-filemtime($cacheFile));
    outputPic($cacheFile,$expire);
    exit();
  }
  // grab the image
  $contents = grab ($imageurl,$contentType);
  if (!$contents) {
    // empty content: return 404
    header('HTTP/1.0 404 Page not found');
    exit();
  }
  
  $tmpfilename =  $cacheDir.genRandString();
  $tmpfilenamehandle = fopen ($tmpfilename,'w+');
  fwrite ($tmpfilenamehandle,$contents);
  fclose ($tmpfilenamehandle);
  //
  chmod ($tmpfilename,0777);
  
  $fullname = $tmpfilename ;

  switch ($contentType) {
    case 'image/gif':
      $image = imagecreatefromgif($fullname);
      break;
    case 'image/jpeg':
      $image = imagecreatefromjpeg($fullname);
      break;
    case 'image/png':
      $image = imagecreatefrompng($fullname);
      break;
    case 'image/bmp':
    case 'image/x-bmp':
    // untested yet
      $image = imagecreatefromwbmp($fullname);
      break;
    default:
      // not an image: return 404
      header('HTTP/1.0 404 Page not found');
      exit();
  }
    // Get image dimensions
  $width = imagesx ($image);
  $height = imagesy ($image);

  $crop=-1;
  // Get new dimensions & crop
  parseResizeExpr($resizeExpr, $new_width, $new_height,$crop);
  // limits check
  if ($new_height>MAX_HEIGHT) $new_height=MAX_HEIGHT;
  if ($new_width>MAX_WIDTH) $new_width = MAX_WIDTH;
  // non ingrandiamo l'immagine
  if ($new_height>$height) $new_height = $height;
  if ($new_width>$width) $new_width = $width;
  $ratio = $width/$height;
  // compute the unspecified dimension
  if ($new_width < 1) {
    $new_width = $new_height * $ratio;
  } elseif ($new_height < 1) {
    $new_height = $new_width/$ratio;
  }
  $new_ratio = $new_width/$new_height;
  $widthRatio = $new_width/$width;
  $heightRatio = $new_height/$height;
  if ($crop==-1) {
    // keep original ratio
    if ($new_ratio != $ratio) {
      if ($widthRatio > $heightRatio) {
        $new_width = $new_height*$ratio;
      } else {
        $new_height = $new_width/$ratio; 
      }
    }
    $src_x=0;
    $src_y=0;
  } else {
    // cropping needs 2 dimensions
    $rratio = $ratio/$new_ratio;
    if ($rratio>1) {
    // horizontal crop
      $centerx   = ($width/2);
      $destwidth = $new_width/$heightRatio;
      $rangex = $centerx-$destwidth/2;
      // crop shift
      // normalize crop between -1 and +1
      // 0-> - 1 SX
      // 50->  0 Center
      // 100-> 1 DX
      $crop=($crop-50)/50;
      $src_x=$centerx-($destwidth/2)+($rangex*$crop);
      $src_y = 0;
      $width = $width/$rratio;
    } else {
    // vertical crop
      $centery   = ($height/2);
      $destheight = $new_height/$widthRatio;
      $rangey = $centery-$destheight/2;
      // crop shift
      $crop=($crop-50)/50;
      $src_y=$centery-($destheight/2)+($rangey*$crop);
      $src_x = 0;
      $height = $height*$rratio;
    }
  }
  // Resample
  $image_p = imagecreatetruecolor($new_width, $new_height);
  imagecopyresampled($image_p, $image, 0, 0, $src_x, $src_y, $new_width, $new_height, $width, $height);
  // Cache thumb
  imagejpeg($image_p, $cacheFile, 85);
  chmod ($cacheFile,0777);
  // Output
  outputPic($cacheFile,$expire);
  unlink ($tmpfilename);
}

function outputPic($file,$expire) {
  
  $if_modified_since = isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ?
      stripslashes($_SERVER['HTTP_IF_MODIFIED_SINCE']) :
      false;
  if ($if_modified_since) {
    $if_modified_since_time = strtotime($if_modified_since);
    // error_log('got if_modified_since ' . $if_modified_since_time . ' ' . filemtime($file));
    if (filemtime($file) <= $if_modified_since_time) {
      // error_log('exiting...');
      header('HTTP/1.0 304 Not Modified');
      exit();
    }
  }

  header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($file)) . ' GMT');
  $expirationTime = time() + $expire;
  header('Expires: ' . gmdate('D, d M Y H:i:s', $expirationTime) . ' GMT');
  header('Cache-Control: max-age='.$expire);
  header('Content-Type: image/jpeg');
  header('Content-Length: '. filesize($file));
  readfile($file);
}

function getParameterValue ($params,$name,$default=null,$default1=null){
  if ($params && array_key_exists ($name,$params)){
    if ($default1==null) {
      $value = $params[$name];
    } else {
      $value = $default1;
    }
  } else $value = $default;
  return $value;
}

?>